from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.conf import settings
from django.contrib.auth.decorators import login_required as django_login_required

def render(template_name, login_required=False):
	def renderer(func):
		def wrapper(request, *args, **kw):
			output = func(request, *args, **kw)
			if not isinstance(output, dict) and output is not None:
				return output
			return render_to_response(template_name, output, context_instance=RequestContext(request))
		if login_required:
			return django_login_required(login_url=settings.LOGIN_URL)(wrapper)
		else:
			return wrapper
	return renderer
   
def add_help_text_headers(func):
	def wrapper(cls, *args, **kw):
		if not hasattr(cls, '_help_text_headers'):
			ht = {}
			for f in cls._meta.get_fields_with_model():
				ht[f[0].name] = f[0].help_text
			cls._help_text_headers = ht
			
		ret = func(cls, *args, **kw)
		if type(ret) is tuple:
			ret[0].headers = cls._help_text_headers
		else:
			ret.headers = cls._help_text_headers
		return ret
	return wrapper

def limit_offset(page):
	if page:
		page = int(page)
	else:
		page = 0
	return settings.PER_PAGE, page * settings.PER_PAGE

def add_prev_next_page(rows, page, next_exists=True):
	page = int(page) if page else 0
		
	prev_page = page - 1 if page > 0 else None
	next_page = page + 1 if len(rows) >= settings.PER_PAGE and next_exists else None
	
	return rows, prev_page, next_page
	
