# -*- coding: utf-8 -*-

from models_soap import CustomersAPI
from models_soap import ProductsAPI
from models_soap import ProductGroupsAPI
from models_soap import OrdersAPI

from helpers import *
import datetime

from django.db import models

class Products:
	@classmethod
	def get(cls, name=None, key=None, code=None, group=None, page=None, do_cache_groups=True, cached_groups={}, session_id='empty'):
		products, next = ProductsAPI.get(name, key, code, group, do_cache_groups=do_cache_groups, cached_groups=cached_groups, session_id=session_id, *limit_offset(page))
		return products, page, next

	@classmethod
	def get_by_key(cls, key, session_id='empty'):
		product = ProductsAPI.get_by_key(key, session_id)
		if not product:
			raise NameError('No such product')
		return product

class ProductGroups:
	@classmethod
	def get(cls, key, session_id='empty'):
		return ProductGroupsAPI.get(key=key, session_id=session_id)

	@classmethod	
	def list(cls, group, session_id='empty'):
		if not group:
			groups = ProductGroupsAPI.top(session_id=session_id)
		else:
			groups = ProductGroupsAPI.kids_of(group, session_id=session_id)
		return groups
	
class Customers:
	@classmethod
	def get(cls, key, session_id='empty'):
		return CustomersAPI.get(key, session_id=session_id)
	
	@classmethod
	def authenticate(cls, username, password, session_id='empty'):
		return CustomersAPI.authenticate(username, password, session_id=session_id)
	
	def is_authenticated(self):
		"""is always true - out of DB auth"""
		return True
	
	#@classmethod
	#def find(cls, key, name, nip):
	#	return CustomersAPI.find(key, name, nip)
	
	
class Orders:
	@classmethod
	def get(cls, user=None, page=None, session_id='empty'):
		orders, next = OrdersAPI.get(user, session_id=session_id, *limit_offset(page))
		return orders, page, next
	
	@classmethod
	def get_by_id(cls, id, session_id='empty'):
		return OrdersAPI.get_by_id(id, session_id=session_id)
	
	@classmethod
	def create(cls, cart, description, session_id='empty'):
		order = OrdersAPI.create(cart.all(), description, session_id=session_id)
		order_result = order.save()
		return order

class Cart(models.Model):
	def __init__(self):
		self.positions = []
	
	def clear(self):
		self.positions = []
	
	def update_amount(self, idx, amount):
		print idx
		position = self.positions[idx]
		position.quantity = amount
		return position
	
	def remove(self, idx):
		print idx
		# cut off position
		npositions = self.positions[:idx] # before the removed product
		for prod in self.positions[idx+1:]: # after the removed product
			prod.idx -= 1 # decrease idx
			npositions.append(prod) # append to npositions
		self.positions = npositions
	
	def all(self):
		return self.positions
	
	def sum(self):
		total = 0.0
		for prod in self.all(): total += prod.set_total()
		return total

	def add_product(self, key, quantity, session_id='empty'):
		product = Products.get_by_key(key, session_id=session_id)
		cart_position = CartPosition()
		cart_position.productkey = product.key
		cart_position.chainkey = product.defChainKey
		cart_position.name = product.name
		cart_position.price = product.price
		cart_position.quantity = quantity
		cart_position.realizationdate = datetime.datetime.now() + datetime.timedelta(days=14) 
		cart_position.idx = len(self.positions)
		self.positions.append(cart_position)
		#cart_position.save()
		return cart_position
	
class CartPosition(models.Model):
	idx = models.IntegerField(help_text="numer")
	productkey = models.TextField(help_text="indeks")
	name = models.TextField(help_text="nazwa")
	price = models.FloatField(null=True, help_text="cena")
	quantity = models.FloatField(help_text="ilość")
	chainkey = models.CharField(max_length=30, help_text="jedn.")
	realizationdate = models.DateTimeField(null=True, help_text='termin realizacji')

	def set_total(self):
		self.total = float(self.price or '0.0') * float(self.quantity)
		return self.total
	

