# -*- coding: utf-8 -*-
from django.http import HttpResponseRedirect
from models import *
from forms import *
from helpers import render
from django.contrib.auth import authenticate, login as django_login, logout as django_logout
from django.shortcuts import redirect
import datetime

### login ###

@render('login.html')
def login(request):
	next = request.REQUEST.get('next', '/')		# return page
	
	if request.method == 'POST':
		user = authenticate(
			username=request.POST['username'],
			password=request.POST['password']
		)

		if not user:
			# check API Server is on ? if not exception will be raised and 500.html shown
			cst = Customers.authenticate(request.POST['username'], request.POST['password'])
			return {'next': next, 'error': 'Błędny login lub hasło'}
		
		request.session['customer_key'] = user.key
		request.session['woe_api_session_id'] = user.woe_api_session_id
		request.session['cart'] = Cart()
		request.session['cached_groups'] = {}
		django_login(request, user)
		return HttpResponseRedirect(next)

	return {'next': next}

@render('logout.html')
def logout(request):
	django_logout(request)
	return {}


### products ###

@render('products.html', login_required=True)
def products(request, group='0', page=None):
	form, products, prev_page, next_page, name, key, code = None, None, None, None, None, None, None

	request.session['group'] = group
	if group != '0':
		product_group = ProductGroups.get(group, session_id=request.session['woe_api_session_id'])
	else:
		product_group = None
		group = None
	#product_groups = ProductGroups.list(group, do_cache_groups=True, cached_groups=request.session['cached_groups'], session_id=request.session['woe_api_session_id'])
	product_groups = ProductGroups.list(group, session_id=request.session['woe_api_session_id'])

	if request.method == 'POST':
		if 'reset' in request.POST:
			if 'postdata' in request.session:
				del request.session['postdata']
		form = ProductsForm(request.POST)
		request.session['postdata'] = request.POST
		
	else:
		if 'postdata' in request.session:
			if page != None or group != None:
				form = ProductsForm(request.session['postdata'])
			else:
				del request.session['postdata']
		"""
		if not page:
			page = request.session.get('products_page')
		"""
	
	request.session['products_page'] = page if page else 0
	
	if not form:
		form = ProductsForm()
		
	else: # if form
		if form.is_valid():
			name = form.cleaned_data['name']
			key = form.cleaned_data['key']
			code = form.cleaned_data['code']
			
			
	products, prev_page, next_page = add_prev_next_page(*Products.get(name, key, code, group, page, do_cache_groups=True, cached_groups=request.session['cached_groups'], session_id=request.session['woe_api_session_id']))
	
	show_search = products or (not products and (name or key or code))
	group_key = '0'
	if product_group:
		group_key = product_group.key
	
	return {'form': form, 'group': product_group, 'group_key': group_key, 'products': products, 'product_groups': product_groups,
			'prev_page': prev_page, 'next_page': next_page, 'show_search': show_search,
			'two_table_headers': len(products) >= settings.PER_PAGE / 2 }



### cart ###

@render('cart.html', login_required=True)
def cart(request):
	errors = []
	error_fields = {}
	cart = request.session.get('cart')
	if request.method == 'POST':
		for idx, amount in request.POST.iteritems():
			idx = idx.split('p')
			if idx[0] != '':
				continue
			try:
				amount = float(amount.replace(",", "."))
			except ValueError, e:
				errors += [u"Wpisz poprawną ilość"]
				error_fields[long(idx[1])] = amount
				continue
			cart.update_amount(long(idx[1]), amount)

	citems = cart.all()
	for p in citems:
		p.quant = error_fields.get(p.idx) or p.quantity
	
	return {'cart': citems, 'cart_sum': cart.sum(), 'two_table_headers': len(citems) >= settings.PER_PAGE / 2, 
		'errors': errors, 'error_fields': error_fields}


@render('cart.html', login_required=True)
def cart_new(request, id=None):
	errors = []
	amount = None
	cart = request.session.get('cart')
	if request.method == 'POST':
		edit = request.POST.get('edit', False)
		try:
			amount = float(request.POST.get('amount').replace(",", "."))
		except ValueError, e:
			amount = None
			errors += ['Wpisz poprawną ilość']
	
		if amount <= 0:
			errors += ['Wpisz ilość większą od zera']
		elif not edit:
			citem = cart.add_product(id, amount, session_id=request.session['woe_api_session_id'])
			group = request.session['group']
			return redirect('products', page=request.session.get('products_page', 0), group=request.session.get('group', '0'))
		
	return {'cart': cart.all(), 'cart_sum': cart.sum(), 
		'amount': amount, 'new_product': Products.get_by_key(id, session_id=request.session['woe_api_session_id']), 'id': id, 'errors': errors}

@render(None, login_required=True)
def cart_remove(request, id=None):
	if id:
		cart = request.session.get('cart')
		#print "Cart remove: ", long(id), "\n"
		cart.remove(long(id))
	return redirect('cart')


@render(None, login_required=True)
def cart_clear(request):
	cart = request.session.get('cart')
	cart.clear()
	return redirect('cart')


	
### order ###

@render('order_new.html', login_required=True)
def order_new(request, client=None):
	client = request.session.get('customer_key')
	cart = request.session.get('cart')
	
	client_data = u"%s %s" % ('_name_', '_nip_')
	errors = []
	
	if not request.method == 'POST':
		form = OrderForm()
		
	else:
		form = OrderForm(request.POST)
		if form.is_valid():
			form.cleaned_data['customerKey'] = client
			description = form.cleaned_data['description']
			order = Orders.create(cart, description, session_id=request.session['woe_api_session_id'])
			if order and order.order_result and order.order_result.is_success():
				cart.clear()
				request.session['order_created_ok'] = 'yes'
				return redirect('orders')
			else:
				errors += ['Zamówienie nie zostało złożone']
				if order and order.order_result:
					errors += [order.order_result.error_message()]
				
	return {'cart': cart.all(), 'cart_sum': cart.sum(), 
		'form': form, 'client': client, 
		'client_data': client_data, 'errors': errors}


@render('orders.html', login_required=True)
def orders(request, page=None):
	orders, prev_page, next_page = add_prev_next_page(*Orders.get(request.user.id, page, session_id=request.session['woe_api_session_id']))
	
	for order in orders:
		order.calc_totals() # generate totals attributes
	
	order_created_ok = False
	if 'order_created_ok' in request.session:
		order_created_ok = True
		del request.session['order_created_ok']
	
	return {'orders': orders, 'next_page': next_page, 'prev_page': prev_page, 'order_created_ok': order_created_ok,
			'two_table_headers': len(orders) >= settings.PER_PAGE / 2 }


@render('order.html', login_required=True)
def order(request, id):
	order = Orders.get_by_id(id, session_id=request.session['woe_api_session_id'])
	order.calc_totals() # generate totals attributes
	positions = order.positions
	return {'order_id': id, 'order': order, 'positions': positions,
			'two_table_headers': len(positions) >= settings.PER_PAGE / 2 }



### client ###

@render('client_select.html', login_required=True)
def client_select(request, key=None):
	if key:
		return redirect('order_new', client=key)
	
	clients = None
	
	if request.method == 'POST':
		form = CustomerFindForm(request.POST)
		if form.is_valid():
			key = form.cleaned_data['customer_key']
			name = form.cleaned_data['customer_name']
			nip = form.cleaned_data['customer_nip']
			clients = Customers.find(key, name, nip, session_id=request.session['woe_api_session_id'])
			print clients
			
	else:
		form = CustomerFindForm()	
	
	
	return {'form':form, 'clients': clients}
